/*
 * This is a very simple C program used to probe the PCI bus, and compare the
 * vendor and model ID of every card found against the pcitable ASCII
 * database. From there it pulls the correct module name, and prints all of
 * the module names that were found for your system.
 *
 * Written by: Greg Kurtzer <greg@caosity.org>
 * License: GPL
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>
#include <linux/pci_ids.h>

#include "config.h"
#include "detect.h"

char ** get_module_from_kern(long long vendor_id, long long device_id, long long class_id) {
	FILE *fd;
	static char class_name[BUFFSIZE];
	char shown_mods[BUFFSIZE];
	char modulename[BUFFSIZE];
	long long vendor, device, subvendor, subdevice, class, class_mask;
	int driver_data;
	char mod_test[BUFFSIZE];
	char modules_pcitable[BUFFSIZE];
	int count = 0;

	char **ret;
	ret = calloc(30, BUFFSIZE);


        struct utsname unameinfo;
	uname(&unameinfo);

	sprintf(modules_pcitable, "/lib/modules/%s/modules.pcimap", unameinfo.release);

        if ((fd = fopen(modules_pcitable, "r"))  == NULL ) {
               	printf("could not open %s!\n", modules_pcitable);
               	exit(1);
        }

        while(!feof(fd)) {
		fscanf(fd, "%s %Lx %Lx %Lx %Lx %Lx %Lx %x", 
			modulename,
			&vendor, 
			&device, 
			&subvendor, 
			&subdevice,
			&class, 
			&class_mask,
			&driver_data
		);


		if ((( vendor == vendor_id && device == device_id ) ||
		     ( (int)vendor == -1 && (int)device == -1 && class == class_id )) &&
		     ! check_module_ignore(modulename)) {

			sprintf(modulename, "%s", check_module_update(modulename));

			strcpy(class_name, get_class(class_id));

			sprintf(mod_test, "@%s@", modulename);
			if ( ! strstr(shown_mods, mod_test) ) {
				ret[count] = strdup(modulename);
				ret[strlen(ret[count])+1] = NULL;
				count++;
			}
			strcat(shown_mods, mod_test);
		}
	}
	fclose(fd);
	ret[count] = NULL;
	return(ret);
}



char ** get_module_from_map(long long vendor_id, long long device_id, long long class_id) {
	FILE *fd;
	static char class_name[BUFFSIZE];
	char shown_mods[BUFFSIZE];
	char modulename[BUFFSIZE];
	long long vendor, device, subvendor, subdevice, class, class_mask;
	int driver_data;
	char mod_test[BUFFSIZE];
	char modules_pcitable[BUFFSIZE];
	int count = 0;

	char **ret;
	ret = calloc(30, BUFFSIZE);


	sprintf(modules_pcitable, "/etc/detect/pcimap");

        if ((fd = fopen(modules_pcitable, "r"))  == NULL ) {
		return(ret);
        }

        while(!feof(fd)) {
		fscanf(fd, "%s %Lx %Lx %Lx %Lx %Lx %Lx %x", 
			modulename,
			&vendor, 
			&device, 
			&subvendor, 
			&subdevice,
			&class, 
			&class_mask,
			&driver_data
		);


		if ((( vendor == vendor_id && device == device_id ) ||
		     ( (int)vendor == -1 && (int)device == -1 && class == class_id )) &&
		     ! check_module_ignore(modulename)) {

			sprintf(modulename, "%s", check_module_update(modulename));

			strcpy(class_name, get_class(class_id));

			sprintf(mod_test, "@%s@", modulename);
			if ( ! strstr(shown_mods, mod_test) ) {
				ret[count] = strdup(modulename);
				ret[strlen(ret[count])+1] = NULL;
				count++;
			}
			strcat(shown_mods, mod_test);
		}
	}
	fclose(fd);
	ret[count] = NULL;
	return(ret);
}



